@@ -31,7 +31,7 @@ no warnings;
 our @ISA = qw( Exporter );
 our @EXPORT = qw( interlinear, interstart );
 
-$VERSION = '0.009';
+$VERSION = '0.011';
 $ABSTRACT = 'Interlinear is a program for building metamodels from incomplete, multivariate, discrete dataseries.';
 
 #######################################################################
@@ -40,7 +40,7 @@ $ABSTRACT = 'Interlinear is a program for building metamodels from incomplete, m
 
 
 my $maxloops= 1000;
-my $sourcefile = "/home/luca/int/starcloud_small.csv";
+my $sourcefile = "/home/luca/int/starcloud.csv";
 my $newfile = $sourcefile . "_new.csv";
 my $report = $newfile . "_report.txt";
 my @mode = ( "wei" ); # #"wei" is weighted gradient linear interpolation of the nearest neighbours.
@@ -59,7 +59,7 @@ my $nearconcurrencies = 1; #Requested minimum number of concurrencies for the ne
 my $parconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each parameter of each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
 my $instconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
 # my %factlevels = ( pairs => { 1=>3, 2=>3, 3=>3, 4=>3, 5=>3 } ); #The keys are the factor numbers and the values are the number of levels in the data series. OBSOLETE.
-my $minreq_forgrad = [1, 1, 7 ]; #THIS VALUES SPECIFY THE NUMBER OF PARAMETER DIFFERENCES (INTEGER NUMBERS) TELLING HOW WELL-ROOTED IN SIMULATED REALITY (I.E, NEAR FROM IT) A POINT MUST BE TO SELECT IT FOR CALCULATING THE GRADIENTS FOR THE METAMODEL. THEY ARE INTEGERS STARTING FROM 1 OR GREATER.
+my $minreq_forgrad = [1, 1, 1 ]; #THIS VALUES SPECIFY THE NUMBER OF PARAMETER DIFFERENCES (INTEGER NUMBERS) TELLING HOW WELL-ROOTED IN SIMULATED REALITY (I.E, NEAR FROM IT) A POINT MUST BE TO SELECT IT FOR CALCULATING THE GRADIENTS FOR THE METAMODEL. THEY ARE INTEGERS STARTING FROM 1 OR GREATER.
 # THE FIRST VALUE IS RELATIVE TO THE FACTORS AND TELLS HOW RELAXED A SEARCH IS.
 # THE SECOND VALUE IS RELATIVE TO THE LEVELS. ONE MAY WANT TO KEEP IT TO 1: THE GRADIENTS ARE CALCULATED USING ONLY ADJACENT INSTANCES.
 # ONE MAY WANT TO KEEP IT TO 1: THE GRADIENTS ARE CALCULATED USING ONLY ADJACENT INSTANCES.
@@ -72,7 +72,7 @@ my $minreq_forcalc = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIAB
 my $minreq_formerge = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR MERGING IT IN THE CALCULATIONS FOR MERGING IT IN THE METAMODEL. A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0 (SIMULATED POINTS AND "META"POINTS WITH THE SAME RIGHT ) MUST BE SPECIFIED. If 0, no entry barrier.
 my $minimumcertain = 0; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED TO USE A DATUM TO BUILD UPON IT. IT DEPENDS ON THE DISTANCE FROM THE ORIGINS OF THE DATUM. THE LONGER THE DISTANCE, THE SMALLER THE STRENGTH (WHICH IS INDEED INVERSELY PROPORTIONAL). A STENGTH VALUE OF 1 IS OF A SIMULATED DATUM, NOT OF A DERIVED DATUM. If 0, no entry barrier.
 my $minimumhold = 1; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED FOR NOT AVERAGING A DATUM WITH ANOTHER, DERIVED DATUM. USUALLY IT HAS TO BE KEPT EQUAL TO $minimimcertain.  If 1, ONLY THE MODEL DATA ARE NOT SUBSTITUTABLE IN THE METAMODEL.
-my $condweight = "yes"; # THIS CONDITIONS TELLS IF THE STRENGTH (LEVEL OF RELIABILITY) OF THE GRADIENTS HAS TO BE CUMULATIVELY TAKEN INTO ACCOUNT IN THE WEIGHTING CALCULATIONS.
+my $condweight = "no"; # THIS CONDITIONS TELLS IF THE STRENGTH (LEVEL OF RELIABILITY) OF THE GRADIENTS HAS TO BE CUMULATIVELY TAKEN INTO ACCOUNT IN THE WEIGHTING CALCULATIONS.
 
 #######################################################################
 
@@ -115,7 +115,7 @@ sub union
   return @union;
 }
 
-sub diff
+sub diff__
 {
   my $aref = shift;
   my $bref = shift;
@@ -149,7 +149,7 @@ sub diff_old
 }
 
 
-sub diff_new
+sub diff
 {
   my $aref = shift;
   my $bref = shift;
@@ -182,7 +182,8 @@ sub adjustmode
 
 sub preparearr
 {
-  my ( $lines_ref, $optformat ) = @_;
+  my ( $lines_ref ) = @_;
+  my $optformat = "yes";
   my @lines = @{ $lines_ref };
   if ( $lines[1] =~ /_/ )
   {
@@ -263,8 +264,6 @@ sub pythagoras
 }
 
 
-
-
 sub weightvals1
 {
   my ( $elt1, $boxgrads_ref, $boxdists_ref, $boxors_ref, $minreq_forcalc, $factbag_ref, $levbag_ref, $stepbag_ref, $elt2, $factlevels_ref, $maxdist, $el3, $elt3, $condweight ) = @_;
@@ -277,7 +276,6 @@ sub weightvals1
   my %factlevels = %{ $factlevels_ref }; #say $tee "weightvals12 \%factlevels: " . dump( %factlevels );
   #say $tee "weightvals1 \$maxdist: " . dump( $maxdist );
 
-
   #if ( ref( $minreq_forcalc ) )
   #{
   #  my $num = $minreq_forcalc->[0];
@@ -331,9 +329,6 @@ sub weightvals1
       $in++;
     }
   }
-  #say $tee "weightvals14out \@factbag: " . dump( @factbag );
-  #say $tee "weightvals14out \@levbag: " . dump( @levbag );
-  #say $tee "weightvals14out \@stepbag: " . dump( @stepbag );
 
   my ( $totdist, $totstrength );
   unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
@@ -347,7 +342,6 @@ sub weightvals1
     $totstrength = ( 1 - $totdist );
   } #say $tee "weightvals15out \$totstrength: " . dump( $totstrength );
 
-
   my $soughtinc = ( $elt2 * $soughtgrad ); #say $tee "weightvals15out \$soughtinc: " . dump( $soughtinc );
 
   if ( ( $soughtinc ne "" ) and ( $totdist ne "" ) and ( $totstrength ne "" ) and ( $totstrength >= $minreq_forcalc ) )
@@ -357,8 +351,6 @@ sub weightvals1
 } #END SUB weightvals1
 
 
-
-
 sub weightvals_merge
 {
   my ( $vals_ref, $strengths_ref, $minreq_formerge, $maxdist ) = @_;
@@ -427,8 +419,6 @@ sub weightvals_merge
 }# END SUB weightvals_merge
 
 
-
-
 sub calcdist
 {
   my ( $el1, $elt1, $factlevels_ref, $maxdist, $el3, $elt3, $condweight ) = @_;
@@ -465,9 +455,6 @@ sub calcdist
       #say $tee "777b \@factbag : " . dump( @factbag ); say $tee "777b \@levbag : " . dump( @levbag ); say $tee "777b \@stepbag : " . dump( @stepbag );
       $c++;
     }
-
-
-
   }
   unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
   {
@@ -485,7 +472,6 @@ sub calcdist
     {
       $strength = ( $strength * $el3 * $elt3 );
     }
-
     return( { dist => $disfactlevelst, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1, diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par } );
   }
 }
@@ -551,13 +537,11 @@ sub calcdistgrad
     {
       $strength = ( $strength * $el3 * $elt3 );
     }
-
     return( { dist => $dist, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1, diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par } );
   }
 }
 
 
-
 sub calcmaxdist
 {
   my ( $arr_ref, $factlevels_ref ) = @_;
@@ -577,19 +561,18 @@ sub calcmaxdist
 }
 
 
-
 sub wei
 {
-  my ( $arr_ref, $arrcopy_ref, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count,
+  my ( $arr_ref, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count,
     $factlevels_ref, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist ) = @_;
   my @arr = @{ $arr_ref };
-  my %arrcopy = %{ $arrcopy_ref };
   #say $tee "ARR VERY_BEFORE: " . dump(@arr);
   my %factlevels = %{ $factlevels_ref };  #say $tee "AND \%factlevels: " . dump( %factlevels );
-  my ( %magic, %wand, %spell, %bank );
+  # my ( %magic, %wand, %spell, %bank );
 
   sub fillbank
   { #say $tee "NOW IN FILLBANK.";
+    my %bank;
     foreach my $el ( @arr )
     { #say $tee "\$el->[1]: " . dump( $el->[1] ); #say $tee "EL: " . dump( $el );
       my $key =  $el->[0] ; #say $tee "\$key: " . dump( $key );
@@ -635,16 +618,14 @@ sub wei
             #  say $tee "NULL!";
             #}
 
-            unless ( !keys %{ $res_ref }
-              #and ( $ordist > 0 ) and ( $ordist ne "" )
-              ) ######## IMPROVE THIS SO AS TO ALLOW VERY DIFFERENT NUMBERS OF LEVELS FOR EACH FACTOR
+            unless ( !keys %{ $res_ref } and ( $ordist > 0 ) and ( $ordist ne "" ) ) ######## IMPROVE THIS SO AS TO ALLOW VERY DIFFERENT NUMBERS OF LEVELS FOR EACH FACTOR
             { #say $tee " SO I AM IN. ";
               my $count = 0;
               foreach my $d10 ( @da1 )
               { #say $tee "\$d10: " . dump( $d10 );
 
                 my $d11 = $da1par[$count]; #say $tee "\$d11: " . dump( $d11 );
-                my $nearness = abs( $d11 - $d21 );
+                #my $nearness = abs( $d11 - $d21 );
 
                 my $co = 0;
                 foreach my $d20 ( @da2 )
@@ -652,28 +633,24 @@ sub wei
                   if ( $d10 == $d20 )
                   {
                     my $stepsize = $factlevels{stepsizes}{$d20}; #say $tee "454\$stepsize: " . dump( $stepsize );
-                    my $d21 = $da2par[$count]; #say $tee "\$d21: " . dump( $d21 );
+                    my $d21 = $da2par[$co]; #say $tee "\$d21: " . dump( $d21 );
                     #if ( ( $d10 eq $d20 ) and ( abs( $d11 - $d21 ) == 1 ) and ( $d11 > 0 ) and ( $d21 > 0 ) and ( $d11 ne "" ) and ( $d21 ne "" )
                     #  and ( $d11 ne $d21 ) ) #### if ( ( $d10 eq $d20 ) and ( $d11 ne $d21 ) ) ###  THE SECOND CONDITION COULD BE LOGICALLY REDUNDANT. CHECK. ####
                     #{
                       my $pair = join( "-", $d11, $d21 );
-                      #my $reversedpair = join( "-", $d21, $d11 );
+                      my @sorted = sort( $d11, $d21 );
+                      my $orderedpair = join( "-", @sorted );
                       my $trio = join( "-", $d10, $pair );
-                      #my $reversedtrio = join( "-", $d10, $reversedpair );
+                      my $orderedtrio = join( "-", $d10, $orderedpair );
 
                       unless ( $trio eq "" )
                       {
                         $bank{$trio}{par} = $d10;
                         push ( @{ $bank{$trio}{trio} }, $trio );
-                        #push ( @{ $bank{$reversedtrio}{trio} }, $reversedtrio );
+                        push ( @{ $bank{$trio}{orderedtrio} }, $orderedtrio );
 
-                        my ( $e, $et);
                         push ( @{ $bank{$trio}{orvals} }, [ $el->[2], $elt->[2] ] );
                         push ( @{ $bank{$trio}{origins} }, $el->[1] );
-                        #push ( @{ $bank{$reversedtrio}{orvals} }, [ $elt->[2], $el->[2] ] );
-                        $e = $el->[2];
-                        $et = $elt->[2];
-
 
                         if ( ( $ordist > 0 ) and ( $ordist ne "" ) and ( $dist ne "" ) and ( $strength ne "" ) )
                         {
@@ -690,8 +667,8 @@ sub wei
 
                         my $pos1 = $d11;
                         my $pos2 = $d21;
-                        my $val1 = $e;
-                        my $val2 = $et;
+                        my $val1 = $el->[2];
+                        my $val2 = $elt->[2];
 
                         my $diffpos = ( $pos1 - $pos2 );
                         my $diffval = ( ( $val2 - $val1 ) / $val1 );
@@ -703,7 +680,14 @@ sub wei
 
                         if ( $grad ne "" )
                         {
-                          push ( @{ $bank{$trio}{grads} }, $grad );
+                          if ( $orderedtrio eq $trio )
+                          {
+                            push ( @{ $bank{$trio}{grads} }, (- $grad ) );
+                          }
+                          else
+                          {
+                            push ( @{ $bank{$trio}{grads} }, $grad );
+                          }
                         }
                       }
                     }
@@ -711,7 +695,6 @@ sub wei
                   $co++;
                 }
                 $count++;
-
               }
             }
           }
@@ -723,11 +706,12 @@ sub wei
 
 
   my $bank_ref = fillbank;
-  #my %bank =  %{ $bank_ref }; say $tee "\%bank: " . dump( %bank );
+  my %bank =  %{ $bank_ref }; say $tee "\%bank: " . dump( %bank );
 
 
   sub cyclearr
-  { #say $tee "NOW IN CYCLEARR.";
+  { say $tee "NTH ARR: " . dump( @arr );
+    my %wand;
     my $coun = 0;
     foreach my $el ( @arr )
     { #say $tee "\$el->[1]: " . dump( $el->[1] ); #say $tee "EL: " . dump( $el );
@@ -767,7 +751,7 @@ sub wei
                     my $d21 = $da2par[$co]; #say $tee "NOW IN2 \$d21: " . dump( $d21 );
                     if ( ( $d10 eq $d20 ) ) #### if ( ( $d10 eq $d20 ) and ( $d11 ne $d21 ) ) ###  THE SECOND CONDITION COULD BE LOGICALLY REDUNDANT. CHECK. ####
                     { #say $tee "NOW IN3";
-                      my $newpair = join( "-", sort ( $d11, $d21 ) );
+                      my $newpair = join( "-", ( $d11, $d21 ) );
                       #my $newunordpair = join( "-", ( $d11, $d21 ) );
 
                       my $newtrio = join( "-", $d10, $newpair ); #say $tee "NEWTRIO: " . dump ( $newtrio );
@@ -808,25 +792,29 @@ sub wei
                       foreach my $e ( @da1par )
                       {
                         if ( $e ne "" )
-                        { #say $tee "NOW IN5"; say $tee "33\$e: " . dump( $e );
-                          #my $e = $da1par[$c];
-                          my $ei = $da2par[$c]; #say $tee "33\$ei: " . dump( $ei );
-                          my $fact = $da1[$c]; #say $tee "33\$fact: " . dump( $fact );
-                          my $diffpar = abs( $e - $ei ); #say $tee "33\$diffpar: " . dump( $diffpar );
-                          #if ( ( $diffpar >= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) )
-                          if ( ( $diffpar <= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) ) ############################à
+                        { say $tee "NOW IN5"; say $tee "33\$e: " . dump( $e );
+                          my $fact = $da1[$c]; say $tee "33\$fact: " . dump( $fact );
+
+                          my $i = 0;
+                          foreach my $ei ( @da2par )
                           {
-                            push ( @factbag, $fact );
-                            push ( @levbag, $diffpar );
-                            push ( @stepbag, $factlevels{stepsizes}{$fact} );
+                            if ( $da1[$c] == $da2[$i] )
+                            {
+                              my $diffpar = abs( $e - $ei ); say $tee "33\$diffpar: " . dump( $diffpar );
+                              #if ( ( $diffpar >= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) )
+                              if ( ( $diffpar <= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) ) ############################à
+                              {
+                                push ( @factbag, $fact );
+                                push ( @levbag, $diffpar );
+                                push ( @stepbag, $factlevels{stepsizes}{$fact} );
+                              }
+                            }
+                            $i++;
                           }
                         }
                         $c++;
                       }
-                      #say $tee "\@factbag: " . dump( @factbag ); #say $tee "\@levbag: " . dump( @levbag ); say $tee "\@stepbag: " . dump( @stepbag );
-
-
-
+                      say $tee "\@factbag: " . dump( @factbag ); say $tee "\@levbag: " . dump( @levbag ); say $tee "\@stepbag: " . dump( @stepbag );
 
                       my ( $soughtinc, $totdist, $totstrength );
 
@@ -857,10 +845,10 @@ sub wei
                         #  say $tee "NOT PUSHING INTO WAND ";
                         #}
                       }
-                      #else
-                      #{
-                      #  say $tee "NOT WEIGHTVALS1ING ";
-                      #}
+                      else
+                      {
+                        say $tee "NOT WEIGHTVALS1ING ";
+                      }
                     }
                   }
                   $co++
@@ -872,11 +860,13 @@ sub wei
         }
       }
     }
+    say $tee "\%wand IN: " . dump( %wand );
+    return( \%wand );
   }
 
-
-  cyclearr;
-  #say $tee "\%wand: " . dump( %wand );
+  my $wand_ref = cyclearr;
+  my %wand = %{ $wand_ref };
+  say $tee "\%wand OUT: " . dump( %wand );
 
 
   my @limb0;
@@ -897,9 +887,8 @@ sub wei
 
 sub purelin
 {
-  my ( $arr_refillf, $arrcopy_ref, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, $factlevels_ref, $maxdist ) = @_;
+  my ( $arr_refillf, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, $factlevels_ref, $maxdist ) = @_;
   my @arr = @{ $arr_ref };
-  my %arrcopy = %{ $arrcopy_ref };
   my ( @linneabagbox );
   my ( %magic, %wand, %spell );
   foreach my $el ( @arr )
@@ -998,12 +987,10 @@ sub purelin
                   }
                   $count++;
                 }
-
               }
               $index++;
             }
           }
-
         }
       }
     }
@@ -1107,7 +1094,6 @@ sub purelin
                   }
                   $count++;
                 }
-
               }
               $index++;
             }
@@ -1140,8 +1126,6 @@ sub purelin
           }
         }
       }
-
-
       unless ( $bagmean eq "" )
       {
         $wand{$ke}{$dee} = $bagmean ;
@@ -1169,9 +1153,8 @@ sub purelin
 
 sub near
 {
-  my ( $arr_ref, $arrcopy_ref, $nearrelaxlimit, $relaxmethod, $overweightnearest, $nearconcurrencies, $count, $factlevels_ref, $maxdist ) = @_;
+  my ( $arr_ref, $nearrelaxlimit, $relaxmethod, $overweightnearest, $nearconcurrencies, $count, $factlevels_ref, $maxdist ) = @_;
   my @arr = @{ $arr_ref };
-  my %arrcopy = %{ $arrcopy_ref };
   my %factlevels = $factlevels_ref;
   foreach my $el ( @arr )
   { #say $tee "\$el->[1]: " . dump( $el->[1] ); #say $tee "EL: " . dump( $el );
@@ -1195,7 +1178,7 @@ sub near
               my ( $da1_ref, $da1par_ref, $da2_ref, $da2par_ref ) = setvalues( \@diff1, \@diff2 );
               my @da1 = @{ $da1_ref };
               my @da2 = @{ $da2_ref };
-ITEMS IN THIS LOOP
+
               my $count = 0;
               foreach my $d10 ( @da1 )
               { #say $tee "\$d10: " . dump( $d10 );
@@ -1213,7 +1196,6 @@ ITEMS IN THIS LOOP
                 }
                 $count++;
               }
-
             }
             $index++;
           }
@@ -1332,12 +1314,14 @@ Name of a configuration file (Unix path):
     It may be specified in the configuration file.
     ";
   }
-  interlinear( $configfile, $sourcefile );
+  @arr = interlinear( $configfile, $sourcefile );
 }
 
 
 ########################################### END SUBS
 
+my $optformat;
+my @arr;
 
 sub interlinear
 {
@@ -1352,9 +1336,8 @@ sub interlinear
   my @lines = <SOURCEFILE>;
   close SOURCEFILE;
 
-  my $optform = "yes";
-
-  my ( $aarr_ref, $optformat ) = preparearr( \@lines, $optform );
+  my $aarr_ref;
+  ( $aarr_ref, $optformat ) = preparearr( \@lines );
 
   my @aarr = @{ $aarr_ref };
 
@@ -1373,22 +1356,20 @@ sub interlinear
     {
       @arr = @aarr;
     }
+    else
+    {
+      @arr = @arr2;
+    }
 
     say $tee "COUNT: " . dump( $count + 1 );
     my $mode__ = $mode[$count] ;
 
-    my %arrcopy;
-    foreach my $el ( @arr )
-    {
-      $arrcopy{$el->[0]} = $el;
-    }
-
     my ( @limbo_wei, @limbo_purelin, @limbo_near, @limbo, %bank, %wand );
 
 
     #if ( ( $mode__ eq "vault" ) or ( $mode__ eq "mix" ) )
     #{
-    #  @limbo_vault = vault( \@arr, \%arrcopy, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev );
+    #  @limbo_vault = vault( \@arr, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev );
     #}
     #say $tee "OBTAINED limbo_vault: " . dump( @limbo_vault );
     #say $tee "THERE ARE " . scalar( @limbo_vault ) . " ITEMS IN THIS LOOP , NUMBER " . ( $count + 1 ). ", 1, FOR GLOBALLY WEIGHTED GRADIENT INTERPOLATION.";
@@ -1396,21 +1377,21 @@ sub interlinear
 
     if ( ( $mode__ eq "wei" ) or ( $mode__ eq "mix" ) )
     {
-      @limbo_wei = wei( \@arr, \%arrcopy, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist );
+      @limbo_wei = wei( \@arr, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist );
       say $tee "THERE ARE " . scalar( @limbo_wei ) . " ITEMS IN THIS LOOP , NUMBER " . ( $count + 1 ). ", 1, FOR WEIGHTED GRADIENT INTERPOLATION OF THE NEAREST NEIGHBOUR.";
     }
     #say $tee "OBTAINED LIMBO_WEI: " . dump( @limbo_wei );
 
     if ( ( $mode__ eq "purelin" ) or ( $mode__ eq "mix" ) )
     {
-      @limbo_purelin = purelin( \@arr, \%arrcopy, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $maxdist );
+      @limbo_purelin = purelin( \@arr, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $maxdist );
       say $tee "THERE ARE " . scalar( @limbo_purelin ) . " ITEMS IN THIS LOOP , NUMBER " . ( $count + 1 ). ", 1, FOR PURE LINEAR INTERPOLATION.";
     }
     #say $tee "OBTAINED LIMBO_PURELIN: " . dump( @limbo_purelin );
 
     if ( ( $mode__ eq "near" ) or ( $mode__ eq "mix" ) )
     {
-      @limbo_near = near( @arr, \%arrcopy, $nearrelaxlimit, $relaxmethod, $overweightnearest, $nearconcurrencies, $count, \%factlev, $maxdist );
+      @limbo_near = near( @arr, $nearrelaxlimit, $relaxmethod, $overweightnearest, $nearconcurrencies, $count, \%factlev, $maxdist );
       say $tee "THERE ARE " . scalar( @limbo_near ) . " ITEMS IN THIS LOOP, NUMBER " . ( $count + 1 ) . ", 2, FOR THE NEAREST NEIGHBOUR STRATEGY.";
     }
     #say $tee "MAGIC: " . dump( %magic );
@@ -1462,7 +1443,7 @@ sub interlinear
           else
           {
             #say $tee "ENTERED0 ";
-            if ( $elt->[3] <= $minimumhold )
+            #if ( $elt->[3] <= $minimumhold )
             { #say $tee "ENTERED1 ";
               my ( $soughtval, $totstrength ) = weightvals_merge( [ $elt->[2], $el->[2] ], [ $elt->[3], $el->[3] ], $minreq_formerge, $maxdist  );
               #say $tee "OBTAINED1 \$soughtval: " . dump( $soughtval ) . "FOR " . dump( $elt->[0] ) ;
@@ -1481,10 +1462,12 @@ sub interlinear
         }
       }
     }
+    @arr2 = @arr ;
     say $tee "INSERTING THE ARRAY UPDATES " . ( $count + 1 ) . " for $sourcefile";
     say $tee "ARR AFTER: " . dump( @arr );
     $count++;
   }
+  return( @arr );
 }
 
 
@@ -1497,43 +1480,36 @@ if ( @ARGV )
   }
   elsif ( $first eq "." )
   {
-    interlinear;
+    @arr = interlinear;
   }
   else
   {
-    interlinear( @ARGV );
+    @arr = interlinear( @ARGV );
   }
 }
 
 open( NEWFILE, ">$newfile" ) or die;
+
 foreach my $entry ( @arr )
 {
-  my $number = scalar( @{ $entry->[1] } );
-  my $coun = 0;
-  foreach my $item ( @{ $entry->[1] } )
+  if ( $optformat eq "yes" )
   {
-    if ( $optformat eq "yes" )
-    {
-      if ( ( not ( $coun == $number ) ) and ( not ( $coun == ( $number -1 ) ) ) )
-      {
-        print NEWFILE "$item" . "_";
-      }
-      else
-      {
-        print NEWFILE "$item" . ",";
-      }
-    }
-    elsif ( $optformat eq "no" )
+    print NEWFILE "$entry->[0],$entry->[2]\n";
+  }
+  elsif ( $optformat eq "no" )
+  {
+    my $coun = 0;
+    foreach my $item ( @{ $entry->[1] } )
     {
       my $response = odd( $coun );
       if ( $response eq "odd" )
       {
         print NEWFILE "$item,";
       }
+      $coun++;
     }
-    $coun++;
+    print NEWFILE "$entry->[2]\n";
   }
-  print NEWFILE "$entry->[2]\n";
 }
 
 
